﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;

namespace ZerimThing
{
    internal partial class frmAddProperty : Form
    {
        private clsProperty NewProperty = null;

        internal clsProperty GetNewProperty()
        {
            return NewProperty;
        }

        internal frmAddProperty(List<clsProperty> iAllProperties)
        {
            InitializeComponent();

            foreach(clsProperty tProperty in iAllProperties)
            {
                ListViewItem tPropertyNameItem = new ListViewItem();

                tPropertyNameItem.Text = tProperty.Name;
                tPropertyNameItem.Tag = tProperty;
                
                lvAllProperties.Items.Add(tPropertyNameItem);
              
            }
        }

        private void lvProperties_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(lvAllProperties.SelectedItems.Count != 1)
            {
                // nothing selected (deselecting everything counts as a "change")
                return;
            }

            // since we only allow selecting one property at a time, take the first selected item
            ListViewItem tSelectedItem = lvAllProperties.SelectedItems[0];

            //--------------------------------------------------------------------------------------------------------
            // safety first! The tag could contain garbage data. I'm kinda paranoid about this though...

            bool tTagDataValid = false;

            // check to make sure it actually has any data at all
            if(tSelectedItem.Tag != null)
            {
                // if it does, check if it's the right type
                if(tSelectedItem.Tag is clsProperty)
                {
                    tTagDataValid = true;
                }
            }

            if(!tTagDataValid)
            {
                // this shouldn't happen so let's assert so the program will stop for us so we can find the problem.
                Debug.Fail("Listview item tag data invalid");

                // for release data, if there's no data or invalid data, let's stop now, it's better to stop than it is to
                // try to display garbage.
                return;
            }

            //----------------------------------------------------------------------------------------------------
            // tag does not contain garbage, let's load the arguments

            // if we didn't verify this earlier and tSelectedItem.Tag wasn't holding a clsProperty, we'd get a cryptic Invalid Cast exception here
            clsProperty tSelectedProperty = (clsProperty) tSelectedItem.Tag;

            int tNumberOfArguments = tSelectedProperty.GetArgumentCount();

            if(tNumberOfArguments == 0)
            {
                // this should not happen
                return;
            }

            // clear out / disable all of the text boxes (the ones this parameter doesn't have should be blank and disabled)
            // whoops I forgot something, do you know what it is?
            
            txtArg1Value.Text = "";
            txtArg1Value.Enabled = false;

            txtArg2Value.Text = "";
            txtArg2Value.Enabled = false;

            txtArg3Value.Text = "";
            txtArg3Value.Enabled = false;

            txtArg4Value.Text = "";
            txtArg4Value.Enabled = false;
            
            if(tNumberOfArguments >= 1)
            {
                txtArg1Value.Text = tSelectedProperty.GetArgumentValueString(0);
                txtArg1Value.Enabled = true;

                lblArg1Name.Text = tSelectedProperty.GetArgumentName(0);
            }         

            // note that there's no "else", it will fall through from 1, to 2, to 3, etc.

            if(tNumberOfArguments >= 2)
            {
                txtArg2Value.Text = tSelectedProperty.GetArgumentValueString(1);
                txtArg2Value.Enabled = true;

                lblArg2Name.Text = tSelectedProperty.GetArgumentName(1);
            }

            if(tNumberOfArguments >= 3)
            {
                txtArg3Value.Text = tSelectedProperty.GetArgumentValueString(2);
                txtArg3Value.Enabled = true;

                lblArg3Name.Text = tSelectedProperty.GetArgumentName(2);
            }

            if(tNumberOfArguments >= 4)
            {
                txtArg4Value.Text = tSelectedProperty.GetArgumentValueString(3);
                txtArg4Value.Enabled = true;

                lblArg4Name.Text = tSelectedProperty.GetArgumentName(3);
            }

        }

        private void btnAddProperty_Click(object sender, EventArgs e)
        {
            if(lvAllProperties.SelectedItems.Count != 1)
            {
                MessageBox.Show("Please select a property to add.");
                return;
            }

            // since we only allow selecting one property at a time, take the first selected item
            ListViewItem tSelectedItem = lvAllProperties.SelectedItems[0];

            //--------------------------------------------------------------------------------------------------------
            // safety first! The tag could contain garbage data. I'm kinda paranoid about this though...
            // note thait this is repeated a few times, it might be better to make this a function...

            bool tTagDataValid = false;

            // check to make sure it actually has any data at all
            if(tSelectedItem.Tag != null)
            {
                // if it does, check if it's the right type
                if(tSelectedItem.Tag is clsProperty)
                {
                    tTagDataValid = true;
                }
            }

            if(!tTagDataValid)
            {
                // this shouldn't happen so let's assert so the program will stop for us so we can find the problem.
                Debug.Fail("Listview item tag data invalid");

                // for release data, if there's no data or invalid data, let's stop now, it's better to stop than it is to
                // try to display garbage.
                return;
            }

            clsProperty tSelectedProperty = (clsProperty) lvAllProperties.SelectedItems[0].Tag;

            this.NewProperty = tSelectedProperty;


            this.DialogResult = DialogResult.OK;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }
    }
}
